@@ -19,6 +19,7 @@ from __future__ import annotations
 import typing
 from typing import Any
 from typing import cast
+from typing import Union
 
 import gc
 import logging
@@ -48,14 +49,16 @@ if typing.TYPE_CHECKING:
     from gajim.common.call_manager import CallManager
     from gajim.common.cert_store import CertificateStore
     from gajim.common.commands import ChatCommands  # noqa: F401
-    from gajim.common.preview import PreviewManager
+    from gajim.common.file_transfer_manager import FileTransferManager
     from gajim.common.storage.archive.storage import MessageArchiveStorage
     from gajim.common.storage.cache import CacheStorage
     from gajim.common.storage.draft import DraftStorage
     from gajim.common.storage.events.storage import EventStorage
+    from gajim.common.task_manager import PulseManager
     from gajim.common.task_manager import TaskManager
 
     from gajim.gtk.application import GajimApplication  # noqa: F401
+    from gajim.gtk.audio_player import AudioPlayer
     from gajim.gtk.main import MainWindow  # noqa: F401
 
 
@@ -64,7 +67,7 @@ version = gajim.__version__
 connections: dict[str, types.Client] = {}
 avatar_cache: dict[str, dict[str, Any]] = {}
 bob_cache: dict[str, bytes] = {}
-services_cache: Any = None
+services_cache: dict[str, Any] = {}
 app = None  # type: GajimApplication
 window = None  # type: MainWindow
 commands = None  # type: ChatCommands
@@ -100,10 +103,11 @@ nicks: dict[str, str] = {}
 cert_store = cast('CertificateStore', None)
 
 call_manager = cast('CallManager', None)
-
-preview_manager = cast('PreviewManager', None)
+audio_player: Union['AudioPlayer', None] = None  # noqa: UP007
+ftm = cast('FileTransferManager', None)
 
 task_manager = cast('TaskManager', None)
+pulse_manager = cast('PulseManager', None)
 
 gupnp_igd = None
 
@@ -131,7 +135,7 @@ def init_process_pool() -> None:
 
     mp_context = multiprocessing.get_context("spawn")
     process_pool = ProcessPoolExecutor(
-        max_workers=4,
+        max_workers=8,
         mp_context=mp_context,
         # https://github.com/python/cpython/issues/115634
         # Start a new process per task to ensure memory is released,
