@@ -10,18 +10,20 @@ import datetime as dt
 import nbxmpp
 import sqlalchemy.exc
 from nbxmpp.namespaces import Namespace
+from nbxmpp.protocol import JID
 from nbxmpp.structs import MessageProperties
 from nbxmpp.structs import StanzaHandler
 from nbxmpp.util import generate_id
 
 from gajim.common import app
 from gajim.common import types
+from gajim.common.const import RETRACTION_FALLBACK
 from gajim.common.events import MessageAcknowledged
 from gajim.common.events import MessageCorrected
 from gajim.common.events import MessageError
 from gajim.common.events import MessageReceived
+from gajim.common.events import MessageRetracted
 from gajim.common.events import MessageSent
-from gajim.common.events import RawMessageReceived
 from gajim.common.events import ReactionUpdated
 from gajim.common.modules.base import BaseModule
 from gajim.common.modules.contacts import GroupchatParticipant
@@ -30,6 +32,8 @@ from gajim.common.modules.message_util import get_chat_type_and_direction
 from gajim.common.modules.message_util import get_eme_message
 from gajim.common.modules.message_util import get_message_timestamp
 from gajim.common.modules.message_util import get_occupant_info
+from gajim.common.modules.message_util import get_reply
+from gajim.common.modules.message_util import get_security_label
 from gajim.common.modules.misc import parse_oob
 from gajim.common.storage.archive import models as mod
 from gajim.common.storage.archive.const import ChatDirection
@@ -59,7 +63,7 @@ class Message(BaseModule):
         self._message_namespaces = {Namespace.ROSTERX, Namespace.IBB}
 
     def _check_if_unknown_contact(self,
-                                  _con: types.xmppClient,
+                                  _con: types.NBXMPPClient,
                                   stanza: nbxmpp.Message,
                                   properties: MessageProperties
                                   ) -> None:
@@ -77,14 +81,15 @@ class Message(BaseModule):
                                                 'ignore_unknown_contacts'):
             return
 
-        jid = properties.jid.bare
-        if self._con.get_module('Roster').get_item(jid) is None:
-            self._log.warning('Ignore message from unknown contact: %s', jid)
+        assert properties.jid is not None
+        bare_jid = properties.jid.new_as_bare()
+        if self._con.get_module('Roster').get_item(bare_jid) is None:
+            self._log.warning('Ignore message from unknown contact: %s', bare_jid)
             self._log.warning(stanza)
             raise nbxmpp.NodeProcessed
 
     def _message_received(self,
-                          _con: types.xmppClient,
+                          _con: types.NBXMPPClient,
                           stanza: nbxmpp.Message,
                           properties: MessageProperties
                           ) -> None:
@@ -103,16 +108,13 @@ class Message(BaseModule):
 
         self._log.info('Received from %s', stanza.getFrom())
 
-        app.ged.raise_event(RawMessageReceived(
-            conn=self._con,
-            stanza=stanza,
-            account=self._account))
-
-        if properties.is_carbon_message and properties.carbon.is_sent:
+        if properties.carbon is not None and properties.carbon.is_sent:
             # Ugly, we treat the from attr as the remote jid,
             # to make that work with sent carbons we have to do this.
             # TODO: Check where in Gajim and plugins we depend on that behavior
-            stanza.setFrom(stanza.getTo())
+            to = stanza.getTo()
+            assert to is not None
+            stanza.setFrom(to)
 
         timestamp = get_message_timestamp(properties)
         remote_jid = properties.remote_jid
@@ -135,7 +137,7 @@ class Message(BaseModule):
         user_delay_ts = None
         if properties.user_timestamp is not None:
             user_delay_ts = dt.datetime.fromtimestamp(
-                properties.user_timestamp, tz=dt.timezone.utc)
+                properties.user_timestamp, tz=dt.UTC)
 
         message_id = properties.id
         if message_id is None:
@@ -163,7 +165,6 @@ class Message(BaseModule):
 
             # Use origin-id because some group chats change the message id
             # on the reflection.
-
             pk = app.storage.archive.update_pending_message(
                 self._account, remote_jid, origin_id, stanza_id)
 
@@ -198,7 +199,7 @@ class Message(BaseModule):
         oob_data = parse_oob(properties)
 
         encryption_data = None
-        if properties.is_encrypted:
+        if properties.encrypted is not None:
             encryption_data = mod.Encryption(
                 **dataclasses.asdict(properties.encrypted))
 
@@ -209,27 +210,9 @@ class Message(BaseModule):
             self._log.debug('Received message without text')
             return
 
-        securitylabel_data = None
-        if properties.has_security_label:
-            assert properties.security_label is not None
-            displaymarking = properties.security_label.displaymarking
-            if displaymarking is not None:
-                securitylabel_data = mod.SecurityLabel(
-                    account_=self._account,
-                    remote_jid_=remote_jid,
-                    label_hash=properties.security_label.get_label_hash(),
-                    displaymarking=displaymarking.name,
-                    fgcolor=displaymarking.fgcolor,
-                    bgcolor=displaymarking.bgcolor,
-                    updated_at=timestamp,
-                )
-
-        reply = None
-        if properties.reply_data is not None:
-            reply = mod.Reply(
-                id=properties.reply_data.id,
-                to=properties.reply_data.to
-            )
+        securitylabel_data = get_security_label(
+            self._account, remote_jid, timestamp, properties.security_label)
+        reply = get_reply(properties.reply_data)
 
         correction_id = None
         if properties.correction is not None:
@@ -274,11 +257,11 @@ class Message(BaseModule):
         app.ged.raise_event(MessageReceived(account=self._account,
                                             jid=remote_jid,
                                             m_type=m_type,
-                                            from_mam=properties.is_mam_message,
+                                            mam=properties.mam,
                                             pk=pk))
 
     def _message_error_received(self,
-                                _con: types.xmppClient,
+                                _con: types.NBXMPPClient,
                                 stanza: nbxmpp.Message,
                                 properties: MessageProperties
                                 ) -> None:
@@ -293,6 +276,8 @@ class Message(BaseModule):
 
         timestamp = get_message_timestamp(properties)
 
+        assert properties.error is not None
+
         error_data = mod.MessageError(
             account_=self._account,
             remote_jid_=remote_jid,
@@ -323,6 +308,7 @@ class Message(BaseModule):
                        ) -> str | None:
 
         if properties.is_mam_message:
+            assert properties.mam is not None
             return properties.mam.id
 
         if not properties.stanza_ids:
@@ -330,7 +316,9 @@ class Message(BaseModule):
 
         if properties.type.is_groupchat:
             archive = properties.remote_jid
+            assert archive is not None
             disco_info = app.storage.cache.get_last_disco_info(archive)
+            assert disco_info is not None
             if not disco_info.supports(Namespace.SID):
                 return None
 
@@ -346,79 +334,17 @@ class Message(BaseModule):
                 return stanza_id.id
         return None
 
-    def build_message_stanza(self, message: OutgoingMessage) -> nbxmpp.Message:
-        own_jid = self._con.get_own_jid()
-        remote_jid = message.contact.jid
-
-        stanza = nbxmpp.Message(
-            to=remote_jid,
-            body=message.get_text(),
-            typ=convert_message_type(message.type)
-        )
-
-        # XEP-0359
-        stanza.setID(message.message_id)
-        stanza.setOriginID(message.message_id)
-
-        # Mark Message as MUC PM
-        if message.is_pm:
-            stanza.setTag('x', namespace=Namespace.MUC_USER)
-
-        # XEP-0444
-        if message.reaction_data is not None:
-            stanza.setReactions(*message.reaction_data)
-            stanza.setTag('store', namespace=Namespace.MSG_HINTS)
-            return stanza
-
-        if message.correct_id:
-            stanza.setTag('replace', attrs={'id': message.correct_id},
-                          namespace=Namespace.CORRECT)
-
-        # XEP-0461
-        if message.reply_data is not None:
-            stanza.setReply(str(message.reply_data.to),
-                            message.reply_data.id,
-                            message.reply_data.fallback_start,
-                            message.reply_data.fallback_end)
-
-        if message.sec_label:
-            stanza.addChild(node=message.sec_label.to_node())
-
-        # XEP-0066
-        if message.oob_url is not None:
-            oob = stanza.addChild('x', namespace=Namespace.X_OOB)
-            oob.addChild('url').setData(message.oob_url)
-
-        # XEP-0184
-        if not own_jid.bare_match(message.contact.jid):
-            if message.has_text() and not message.is_groupchat:
-                stanza.setReceiptRequest()
-
-        # XEP-0085
-        if message.chatstate is not None:
-            stanza.setTag(message.chatstate, namespace=Namespace.CHATSTATES)
-            if not message.has_text():
-                stanza.setTag('no-store',
-                              namespace=Namespace.MSG_HINTS)
-
-        # XEP-0333
-        if message.has_text():
-            stanza.setMarkable()
-        if message.marker:
-            marker, id_ = message.marker
-            stanza.setMarker(marker, id_)
-
-        return stanza
-
     def store_message(self, message: OutgoingMessage) -> None:
         if (not message.has_text() and
-                message.reaction_data is None):
+                message.reaction_data is None and
+                message.retraction_id is None):
             return
 
-        if (message.type == MessageType.GROUPCHAT and
-                message.reaction_data is not None):
-            # Store reaction when the MUC reflects it
-            return
+        if message.type == MessageType.GROUPCHAT:
+            if (message.reaction_data is not None or
+                    message.retraction_id is not None):
+                # Store reaction when the MUC reflects it
+                return
 
         direction = ChatDirection.OUTGOING
         remote_jid = message.contact.jid
@@ -444,7 +370,7 @@ class Message(BaseModule):
 
             occupant = mod.Occupant(
                 account_=self._account,
-                remote_jid_=remote_jid,
+                remote_jid_=muc_jid,
                 id=str(occupant_id),
                 real_remote_jid_=real_jid,
                 nickname=resource,
@@ -456,17 +382,28 @@ class Message(BaseModule):
 
         if message.reaction_data is not None:
             reactions_id, reactions = message.reaction_data
-            reaction = mod.Reaction(
-                account_=self._account,
-                remote_jid_=remote_jid,
-                occupant_=occupant,
-                id=reactions_id,
-                direction=direction,
-                emojis=';'.join(reactions),
-                timestamp=message.timestamp,
-            )
+            if not reactions:
+                app.storage.archive.delete_reaction(
+                    account=self._account,
+                    jid=remote_jid,
+                    occupant_id=occupant.id if occupant is not None else None,
+                    reaction_id=reactions_id,
+                    direction=direction,
+                )
+                self._log.info('Delete own reactions for: %s', reactions_id)
 
-            app.storage.archive.upsert_row2(reaction)
+            else:
+                reaction = mod.Reaction(
+                    account_=self._account,
+                    remote_jid_=remote_jid,
+                    occupant_=occupant,
+                    id=reactions_id,
+                    direction=direction,
+                    emojis=';'.join(reactions),
+                    timestamp=message.timestamp,
+                )
+
+                app.storage.archive.upsert_row2(reaction)
 
             app.ged.raise_event(
                 ReactionUpdated(
@@ -477,31 +414,37 @@ class Message(BaseModule):
             )
             return
 
+        if message.retraction_id is not None:
+            retraction = mod.Retraction(
+                account_=self._account,
+                remote_jid_=remote_jid,
+                occupant_=occupant,
+                id=message.retraction_id,
+                direction=direction,
+                timestamp=message.timestamp,
+            )
+
+            app.storage.archive.insert_row(retraction, ignore_on_conflict=True)
+
+            app.ged.raise_event(
+                MessageRetracted(
+                    account=self._account, jid=remote_jid, retraction=retraction
+                )
+            )
+            return
+
         encryption_data = None
         encryption = message.get_encryption()
         if encryption is not None:
             encryption_data = mod.Encryption(**dataclasses.asdict(encryption))
 
-        securitylabel_data = None
-        if message.sec_label is not None:
-            displaymarking = message.sec_label.displaymarking
-            if displaymarking is not None:
-                securitylabel_data = mod.SecurityLabel(
-                    account_=self._account,
-                    remote_jid_=remote_jid,
-                    label_hash=message.sec_label.get_label_hash(),
-                    displaymarking=displaymarking.name,
-                    fgcolor=displaymarking.fgcolor,
-                    bgcolor=displaymarking.bgcolor,
-                    updated_at=message.timestamp,
-                )
+        securitylabel_data = get_security_label(
+            self._account, remote_jid, message.timestamp, message.sec_label)
+        reply = get_reply(message.reply_data)
 
-        reply = None
+        thread_id = None
         if message.reply_data is not None:
-            reply = mod.Reply(
-                id=message.reply_data.id,
-                to=message.reply_data.to
-            )
+            thread_id = message.reply_data.thread_id
 
         oob_data: list[mod.OOB] = []
         if message.oob_url is not None:
@@ -519,6 +462,7 @@ class Message(BaseModule):
             id=message.message_id,
             stanza_id=None,
             user_delay_ts=None,
+            thread_id_=thread_id,
             correction_id=message.correct_id,
             encryption_=encryption_data,
             oob=oob_data,
@@ -544,3 +488,84 @@ class Message(BaseModule):
                         account=message.account,
                         pk=pk,
                         play_sound=message.play_sound))
+
+
+def build_message_stanza(message: OutgoingMessage, own_jid: JID) -> nbxmpp.Message:
+    remote_jid = message.contact.jid
+
+    stanza = nbxmpp.Message(
+        to=str(remote_jid),
+        body=message.get_text(),
+        typ=convert_message_type(message.type)
+    )
+
+    # XEP-0359
+    stanza.setID(message.message_id)
+    stanza.setOriginID(message.message_id)
+
+    thread_id = None
+
+    # Mark Message as MUC PM
+    if message.is_pm:
+        stanza.setTag('x', namespace=Namespace.MUC_USER)
+
+    # XEP-0444
+    if message.reaction_data is not None:
+        stanza.setReactions(*message.reaction_data)
+        stanza.setTag('store', namespace=Namespace.MSG_HINTS)
+        return stanza
+
+    if message.correct_id:
+        stanza.setTag('replace', attrs={'id': message.correct_id},
+                      namespace=Namespace.CORRECT)
+
+    # XEP-0461
+    if message.reply_data is not None:
+        thread_id = message.reply_data.thread_id
+        stanza.setReply(str(message.reply_data.to),
+                        message.reply_data.id,
+                        message.reply_data.fallback_start,
+                        message.reply_data.fallback_end)
+
+    if message.sec_label:
+        stanza.addChild(node=message.sec_label.to_node())
+
+    # XEP-0066
+    if message.oob_url is not None:
+        oob = stanza.addChild('x', namespace=Namespace.X_OOB)
+        oob.addChild('url').setData(message.oob_url)
+
+    # XEP-0184
+    if not own_jid.bare_match(message.contact.jid):
+        if message.has_text() and not message.is_groupchat:
+            stanza.setReceiptRequest()
+
+    # XEP-0085
+    if message.chatstate is not None:
+        stanza.setTag(message.chatstate, namespace=Namespace.CHATSTATES)
+        if not message.has_text():
+            stanza.setTag('no-store',
+                          namespace=Namespace.MSG_HINTS)
+
+    # XEP-0333
+    if message.has_text():
+        stanza.setMarkable()
+    if message.marker:
+        marker, id_ = message.marker
+        stanza.setMarker(marker, id_)
+
+    # XEP-0424
+    if message.retraction_id is not None:
+        stanza.setRetracted(
+            message.retraction_id, fallback_text=RETRACTION_FALLBACK)
+
+    # XEP-0490
+    if message.mds_id is not None:
+        stanza.setMdsAssist(message.mds_id, own_jid.new_as_bare())
+
+    # Thread
+    # Currently only supported for replies
+    if thread_id is not None:
+        stanza.setThread(thread_id)
+
+    return stanza
