@@ -17,6 +17,7 @@ import sqlalchemy as sa
 from nbxmpp.protocol import JID
 from nbxmpp.structs import CommonError
 from sqlalchemy.exc import IntegrityError
+from sqlalchemy.exc import OperationalError
 from sqlalchemy.orm import DeclarativeBase
 from sqlalchemy.orm import Mapped
 from sqlalchemy.orm import mapped_column
@@ -27,6 +28,7 @@ from gajim.common import app
 from gajim.common.const import Trust
 from gajim.common.events import DBMigrationFinished
 from gajim.common.events import DBMigrationProgress
+from gajim.common.events import DBMigrationStart
 from gajim.common.storage.archive import models as mod
 from gajim.common.storage.archive.const import ChatDirection
 from gajim.common.storage.archive.const import MessageState
@@ -34,7 +36,7 @@ from gajim.common.storage.archive.const import MessageType
 from gajim.common.storage.base import AlchemyStorage
 from gajim.common.storage.base import is_unique_constraint_error
 
-log = logging.getLogger('gajim.c.storage.archive.migration')
+log = logging.getLogger("gajim.c.storage.archive.migration")
 
 
 class MigrationBase(DeclarativeBase):
@@ -42,17 +44,17 @@ class MigrationBase(DeclarativeBase):
 
 
 class LastArchiveMessage(MappedAsDataclass, MigrationBase, kw_only=True):
-    __tablename__ = 'last_archive_message'
+    __tablename__ = "last_archive_message"
 
-    jid_id: Mapped[int] = mapped_column(sa.ForeignKey('jids.jid_id'), primary_key=True)
-    remote: Mapped[Jids | None] = relationship(lazy='joined', viewonly=True, init=False)
+    jid_id: Mapped[int] = mapped_column(sa.ForeignKey("jids.jid_id"), primary_key=True)
+    remote: Mapped[Jids | None] = relationship(lazy="joined", viewonly=True, init=False)
     last_mam_id: Mapped[str | None]
     oldest_mam_timestamp: Mapped[float | None]
     last_muc_timestamp: Mapped[float | None]
 
 
 class Jids(MappedAsDataclass, MigrationBase, kw_only=True):
-    __tablename__ = 'jids'
+    __tablename__ = "jids"
 
     jid_id: Mapped[int] = mapped_column(primary_key=True)
     jid: Mapped[str]
@@ -60,17 +62,17 @@ class Jids(MappedAsDataclass, MigrationBase, kw_only=True):
 
 
 class Logs(MappedAsDataclass, MigrationBase, kw_only=True):
-    __tablename__ = 'logs'
+    __tablename__ = "logs"
 
     log_line_id: Mapped[int] = mapped_column(primary_key=True)
-    account_id: Mapped[int] = mapped_column(sa.ForeignKey('jids.jid_id'))
+    account_id: Mapped[int] = mapped_column(sa.ForeignKey("jids.jid_id"))
     account: Mapped[Jids | None] = relationship(
-        lazy='joined', foreign_keys=[account_id], viewonly=True, init=False
+        lazy="joined", foreign_keys=[account_id], viewonly=True, init=False
     )
 
-    jid_id: Mapped[int] = mapped_column(sa.ForeignKey('jids.jid_id'))
+    jid_id: Mapped[int] = mapped_column(sa.ForeignKey("jids.jid_id"))
     remote: Mapped[Jids | None] = relationship(
-        lazy='joined', foreign_keys=[jid_id], viewonly=True, init=False
+        lazy="joined", foreign_keys=[jid_id], viewonly=True, init=False
     )
 
     contact_name: Mapped[str | None]
@@ -120,6 +122,10 @@ class Migration:
             self._v11_and_v12()
         if user_version < 13:
             self._v13()
+        if user_version < 14:
+            self._v14()
+        if user_version < 15:
+            self._v15()
 
         app.ged.raise_event(DBMigrationFinished())
 
@@ -128,7 +134,25 @@ class Migration:
             for stmt in statements:
                 conn.execute(sa.text(stmt))
 
+    def _execute_multiple_with_error(
+        self, statements: list[tuple[str, str] | tuple[str, None]]
+    ) -> None:
+        with self._engine.begin() as conn:
+            for stmt, error_string in statements:
+                try:
+                    conn.execute(sa.text(stmt))
+                except OperationalError as e:
+                    if error_string is not None:
+                        assert e.orig is not None
+                        sql_error = e.orig.args[0]
+                        if error_string in sql_error:
+                            log.warning('Ignore "%s" for %s', sql_error, e.statement)
+                            continue
+                    raise
+
     def _pre_v7(self, user_version: int) -> None:
+        app.ged.raise_event(DBMigrationStart(version=7))
+
         if user_version == 0:
             # All migrations from 0.16.9 until 1.0.0
             statements = [
@@ -138,13 +162,13 @@ class Migration:
                 'ALTER TABLE logs ADD COLUMN "encryption_state" TEXT',
                 'ALTER TABLE logs ADD COLUMN "marker" INTEGER',
                 'ALTER TABLE logs ADD COLUMN "additional_data" TEXT',
-                '''CREATE TABLE IF NOT EXISTS last_archive_message(
+                """CREATE TABLE IF NOT EXISTS last_archive_message(
                     jid_id INTEGER PRIMARY KEY UNIQUE,
                     last_mam_id TEXT,
                     oldest_mam_timestamp TEXT,
                     last_muc_timestamp TEXT
-                    )''',
-                'PRAGMA user_version=1',
+                    )""",
+                "PRAGMA user_version=1",
             ]
 
             self._execute_multiple(statements)
@@ -152,24 +176,24 @@ class Migration:
         if user_version < 2:
             statements = [
                 (
-                    'ALTER TABLE last_archive_message '
+                    "ALTER TABLE last_archive_message "
                     'ADD COLUMN "sync_threshold" INTEGER'
                 ),
-                'PRAGMA user_version=2',
+                "PRAGMA user_version=2",
             ]
             self._execute_multiple(statements)
 
         if user_version < 3:
             statements = [
                 'ALTER TABLE logs ADD COLUMN "message_id" TEXT',
-                'PRAGMA user_version=3',
+                "PRAGMA user_version=3",
             ]
             self._execute_multiple(statements)
 
         if user_version < 4:
             statements = [
                 'ALTER TABLE logs ADD COLUMN "error" TEXT',
-                'PRAGMA user_version=4',
+                "PRAGMA user_version=4",
             ]
             self._execute_multiple(statements)
 
@@ -177,11 +201,13 @@ class Migration:
             statements = [
                 'ALTER TABLE logs ADD COLUMN "real_jid" TEXT',
                 'ALTER TABLE logs ADD COLUMN "occupant_id" TEXT',
-                'PRAGMA user_version=7',
+                "PRAGMA user_version=7",
             ]
             self._execute_multiple(statements)
 
     def _v8(self) -> None:
+        app.ged.raise_event(DBMigrationStart(version=8))
+
         for account_name in app.settings.get_accounts():
             jid = app.get_jid_from_account(account_name)
             self._accounts[jid] = account_name
@@ -215,25 +241,26 @@ class Migration:
             for archive_row in self._archive.get_session().scalars(stmt):
                 self._process_archive_row(conn, archive_row, account_pks)
 
-            conn.execute(sa.text('DROP TABLE last_archive_message'))
-            conn.execute(sa.text('DROP TABLE logs'))
-            conn.execute(sa.text('DROP TABLE jids'))
-            conn.execute(sa.text('DROP TABLE IF EXISTS unread_messages'))
-            conn.execute(sa.text('PRAGMA user_version=8'))
+            conn.execute(sa.text("DROP TABLE last_archive_message"))
+            conn.execute(sa.text("DROP TABLE logs"))
+            conn.execute(sa.text("DROP TABLE jids"))
+            conn.execute(sa.text("DROP TABLE IF EXISTS unread_messages"))
+            conn.execute(sa.text("PRAGMA user_version=8"))
 
     def _v9(self) -> None:
+        app.ged.raise_event(DBMigrationStart(version=9))
         statements = [
-            'CREATE INDEX IF NOT EXISTS idx_stanza_id ON message(stanza_id, fk_remote_pk, fk_account_pk);',
-            'PRAGMA user_version=9',
+            "CREATE INDEX IF NOT EXISTS idx_stanza_id ON message(stanza_id, fk_remote_pk, fk_account_pk);",
+            "PRAGMA user_version=9",
         ]
 
         self._execute_multiple(statements)
 
     def _v10(self) -> None:
-        self._execute_multiple(['PRAGMA foreign_keys=OFF'])
+        app.ged.raise_event(DBMigrationStart(version=10))
+        self._execute_multiple(["PRAGMA foreign_keys=OFF"])
 
         with self._engine.begin() as conn:
-
             stmt = sa.select(mod.Message.fk_occupant_pk).union(
                 sa.select(mod.Moderation.fk_occupant_pk),
                 sa.select(mod.DisplayedMarker.fk_occupant_pk),
@@ -241,11 +268,14 @@ class Migration:
             occupant_pks: set[int | None] = set(conn.scalars(stmt))
             occupant_pks.discard(None)
 
-            conn.execute(sa.delete(mod.Occupant).where(mod.Occupant.pk.not_in(occupant_pks)))
+            conn.execute(
+                sa.delete(mod.Occupant).where(mod.Occupant.pk.not_in(occupant_pks))
+            )
             conn.commit()
 
-        with self._engine.begin() as conn:
+        app.ged.raise_event(DBMigrationProgress(count=100, progress=50))
 
+        with self._engine.begin() as conn:
             stmt = sa.select(mod.Occupant.fk_remote_pk).union(
                 sa.select(mod.Occupant.fk_real_remote_pk),
                 sa.select(mod.SecurityLabel.fk_remote_pk),
@@ -260,24 +290,68 @@ class Migration:
             occupant_pks: set[int | None] = set(conn.scalars(stmt))
             occupant_pks.discard(None)
 
-            conn.execute(sa.delete(mod.Remote).where(mod.Remote.pk.not_in(occupant_pks)))
+            conn.execute(
+                sa.delete(mod.Remote).where(mod.Remote.pk.not_in(occupant_pks))
+            )
 
-        self._execute_multiple([
-            'PRAGMA foreign_keys=ON',
-            'PRAGMA user_version=10'
-        ])
+        self._execute_multiple(["PRAGMA foreign_keys=ON", "PRAGMA user_version=10"])
 
     def _v11_and_v12(self) -> None:
+        app.ged.raise_event(DBMigrationStart(version=12))
         mod.Base.metadata.create_all(self._engine)
-        self._execute_multiple(['PRAGMA user_version=12'])
+        self._execute_multiple(["PRAGMA user_version=12"])
 
     def _v13(self) -> None:
+        app.ged.raise_event(DBMigrationStart(version=13))
         statements = [
-            'ALTER TABLE occupant ADD COLUMN "blocked" INTEGER NOT NULL DEFAULT (0)',
-            'PRAGMA user_version=13',
+            (
+                'ALTER TABLE occupant ADD COLUMN "blocked" INTEGER NOT NULL DEFAULT (0)',
+                "duplicate column",
+            ),
+            ("PRAGMA user_version=13", None),
+        ]
+        self._execute_multiple_with_error(statements)
+
+    def _v14(self) -> None:
+        app.ged.raise_event(DBMigrationStart(version=14))
+        statements = [
+            (
+                "CREATE INDEX IF NOT EXISTS idx_displayed_marker_last ON displayed_marker "
+                "(fk_remote_pk, fk_account_pk, timestamp DESC)"
+            ),
+            (
+                "CREATE INDEX IF NOT EXISTS idx_displayed_marker_last_gc ON displayed_marker "
+                "(fk_remote_pk, fk_account_pk, fk_occupant_pk, timestamp DESC)"
+            ),
+            "PRAGMA user_version=14",
         ]
         self._execute_multiple(statements)
 
+    def _v15(self) -> None:
+        # In the past some PM messages were not correctly discovered
+        # Find them and set the type and resource
+        app.ged.raise_event(DBMigrationStart(version=15))
+        unique_remotes_stmt = (
+            sa.select(mod.Message.fk_remote_pk).distinct().where(mod.Message.type == 3)
+        )
+        remotes_stmt = sa.select(mod.Remote).where(
+            mod.Remote.pk.in_(unique_remotes_stmt.scalar_subquery())
+        )
+
+        app.ged.raise_event(DBMigrationProgress(count=100, progress=10))
+        with self._archive.get_session() as s:
+            remotes = s.execute(remotes_stmt).scalars().all()
+            for remote in remotes:
+                stmt = (
+                    sa.update(mod.Message)
+                    .where(mod.Message.fk_remote_pk == remote.pk, mod.Message.type != 3)
+                    .values(type=3, resource=remote.jid.resource)
+                )
+                s.execute(stmt)
+            s.commit()
+
+        self._execute_multiple(["PRAGMA user_version=15"])
+
     def _get_account_pks(self, conn: sa.Connection) -> list[int]:
         account_pks: list[int] = []
         for account in app.settings.get_accounts():
@@ -285,7 +359,7 @@ class Migration:
             try:
                 jid = JID.from_string(jid_str)
             except Exception:
-                log.warning('Unable to parse account: %s', jid_str)
+                log.warning("Unable to parse account: %s", jid_str)
                 continue
 
             pk = self._get_account_pk(conn, jid)
@@ -301,7 +375,7 @@ class Migration:
     ) -> None:
         if archive_row.remote is None:
             log.warning(
-                'Unable to migrate mam state because jid_id %s was not found',
+                "Unable to migrate mam state because jid_id %s was not found",
                 archive_row.jid_id,
             )
             return
@@ -309,9 +383,7 @@ class Migration:
         try:
             remote_jid = JID.from_string(archive_row.remote.jid)
         except Exception as error:
-            log.warning(
-                'Invalid JID found, unable to migrate archive state: %s', error
-            )
+            log.warning("Invalid JID found, unable to migrate archive state: %s", error)
             return
 
         remote_pk = self._get_remote_pk(conn, remote_jid)
@@ -347,7 +419,7 @@ class Migration:
                 )
             except IntegrityError:
                 log.warning(
-                    'Unable to migrate mam archive state, because it was already migrated'
+                    "Unable to migrate mam archive state, because it was already migrated"
                 )
                 return
 
@@ -355,19 +427,19 @@ class Migration:
         m_type, direction = KIND_MAPPING[log_row.kind]
 
         if log_row.time is None:
-            log.warning('Unable to migrate message because timestamp is empty')
+            log.warning("Unable to migrate message because timestamp is empty")
             return
 
         if log_row.account is None:
             log.warning(
-                'Unable to migrate message because account_id %s was not found',
+                "Unable to migrate message because account_id %s was not found",
                 log_row.account_id,
             )
             return
 
         if log_row.remote is None:
             log.warning(
-                'Unable to migrate message because jid_id %s was not found',
+                "Unable to migrate message because jid_id %s was not found",
                 log_row.jid_id,
             )
             return
@@ -376,7 +448,7 @@ class Migration:
             account_jid = JID.from_string(log_row.account.jid)
         except Exception:
             log.warning(
-                'Unable to migrate message because of invalid account jid: %s',
+                "Unable to migrate message because of invalid account jid: %s",
                 log_row.account.jid,
             )
             return
@@ -385,7 +457,7 @@ class Migration:
             remote_jid = JID.from_string(log_row.remote.jid)
         except Exception:
             log.warning(
-                'Unable to migrate message because of invalid remote jid: %s',
+                "Unable to migrate message because of invalid remote jid: %s",
                 log_row.remote.jid,
             )
             return
@@ -394,13 +466,13 @@ class Migration:
         remote_pk = self._get_remote_pk(conn, remote_jid)
 
         try:
-            additional_data = json.loads(log_row.additional_data or '{}')
+            additional_data = json.loads(log_row.additional_data or "{}")
         except Exception as error:
             raise ValueError(
-                f'failed to parse additional_data: {error} - {log_row.additional_data}'
+                f"failed to parse additional_data: {error} - {log_row.additional_data}"
             )
 
-        user_timestamp = additional_data.get('user_timestamp')
+        user_timestamp = additional_data.get("user_timestamp")
         user_delay_ts = None
         if user_timestamp is not None:
             user_delay_ts = datetime.fromtimestamp(user_timestamp, UTC)
@@ -410,23 +482,23 @@ class Migration:
         encryption_pk = self._insert_encryption_data(conn, additional_data)
 
         text = log_row.message
-        corrected = additional_data.get('corrected')
+        corrected = additional_data.get("corrected")
         if corrected is not None:
-            text = corrected.get('original_text') or text
+            text = corrected.get("original_text") or text
 
         message_data: dict[str, Any] = {
-            'fk_account_pk': account_pk,
-            'fk_remote_pk': remote_pk,
-            'resource': log_row.contact_name,
-            'type': m_type,
-            'direction': direction,
-            'timestamp': timestamp,
-            'state': MessageState.ACKNOWLEDGED,
-            'id': log_row.message_id,
-            'stanza_id': log_row.stanza_id,
-            'text': text,
-            'user_delay_ts': user_delay_ts,
-            'fk_encryption_pk': encryption_pk,
+            "fk_account_pk": account_pk,
+            "fk_remote_pk": remote_pk,
+            "resource": log_row.contact_name,
+            "type": m_type,
+            "direction": direction,
+            "timestamp": timestamp,
+            "state": MessageState.ACKNOWLEDGED,
+            "id": log_row.message_id,
+            "stanza_id": log_row.stanza_id,
+            "text": text,
+            "user_delay_ts": user_delay_ts,
+            "fk_encryption_pk": encryption_pk,
         }
 
         try:
@@ -434,10 +506,10 @@ class Migration:
                 sa.insert(mod.Message).returning(mod.Message.pk), [message_data]
             ).scalar()
         except IntegrityError as error:
-            if 'message.stanza_id' not in error.args[0]:
+            if "message.stanza_id" not in error.args[0]:
                 raise
 
-            message_data['stanza_id'] = str(uuid.uuid4())
+            message_data["stanza_id"] = str(uuid.uuid4())
 
             pk = conn.execute(
                 sa.insert(mod.Message).returning(mod.Message.pk), [message_data]
@@ -458,7 +530,7 @@ class Migration:
         ).scalar()
 
         if pk is None:
-            raise ValueError('Failed to insert account')
+            raise ValueError("Failed to insert account")
 
         self._account_pks[jid] = pk
         return pk
@@ -473,7 +545,7 @@ class Migration:
         ).scalar()
 
         if pk is None:
-            raise ValueError('Failed to insert remote')
+            raise ValueError("Failed to insert remote")
 
         self._remote_pks[jid] = pk
         return pk
@@ -487,20 +559,20 @@ class Migration:
         if not additional_data:
             return None
 
-        gajim_data = additional_data.get('gajim')
+        gajim_data = additional_data.get("gajim")
         if gajim_data is None:
             return None
 
-        url = gajim_data.get('oob_url')
-        description = gajim_data.get('oob_desc')
+        url = gajim_data.get("oob_url")
+        description = gajim_data.get("oob_desc")
 
         if url is None:
             return None
 
         oob_data = {
-            'fk_message_pk': fk_message_pk,
-            'url': url,
-            'description': description,
+            "fk_message_pk": fk_message_pk,
+            "url": url,
+            "description": description,
         }
 
         conn.execute(sa.insert(mod.OOB), [oob_data])
@@ -533,15 +605,15 @@ class Migration:
             return None
 
         error_data = {
-            'fk_account_pk': fk_account_pk,
-            'fk_remote_pk': fk_remote_pk,
-            'message_id': log_row.message_id,
-            'by': by,
-            'type': e_type,
-            'text': text,
-            'condition': condition,
-            'condition_text': condition_text,
-            'timestamp': timestamp + timedelta(seconds=1),
+            "fk_account_pk": fk_account_pk,
+            "fk_remote_pk": fk_remote_pk,
+            "message_id": log_row.message_id,
+            "by": by,
+            "type": e_type,
+            "text": text,
+            "condition": condition,
+            "condition_text": condition_text,
+            "timestamp": timestamp + timedelta(seconds=1),
         }
 
         try:
@@ -556,19 +628,19 @@ class Migration:
         if not additional_data:
             return None
 
-        encrypted = additional_data.get('encrypted')
+        encrypted = additional_data.get("encrypted")
         if encrypted is None:
             return None
 
-        protocol = encrypted.get('name')
-        key = encrypted.get('fingerprint')
-        trust = encrypted.get('trust')
+        protocol = encrypted.get("name")
+        key = encrypted.get("fingerprint")
+        trust = encrypted.get("trust")
 
-        if protocol not in ('OpenPGP', 'OMEMO', 'PGP'):
+        if protocol not in ("OpenPGP", "OMEMO", "PGP"):
             return None
 
         if key is None:
-            key = 'Unknown'
+            key = "Unknown"
 
         if trust is None:
             trust = Trust.VERIFIED
@@ -579,7 +651,7 @@ class Migration:
             except Exception:
                 trust = Trust.UNTRUSTED
 
-        encryption_data = {'protocol': protocol, 'key': key, 'trust': trust}
+        encryption_data = {"protocol": protocol, "key": key, "trust": trust}
         encryption_pk = self._encryption_pks.get(tuple(encryption_data.values()))
         if encryption_pk is not None:
             return encryption_pk
