@@ -28,11 +28,13 @@ from __future__ import annotations
 from typing import Any
 
 import os
+import shutil
 import sys
 from collections.abc import Callable
 from datetime import datetime
 from datetime import timedelta
 from datetime import UTC
+from pathlib import Path
 
 from gi.repository import Adw
 from gi.repository import Gio
@@ -53,11 +55,11 @@ from gajim.common.const import GAJIM_FAQ_URI
 from gajim.common.const import GAJIM_PRIVACY_POLICY_URI
 from gajim.common.const import GAJIM_SUPPORT_JID
 from gajim.common.const import GAJIM_WIKI_URI
+from gajim.common.helpers import dump_json
 from gajim.common.helpers import load_json
 from gajim.common.i18n import _
 from gajim.common.modules.contacts import BareContact
 from gajim.common.modules.contacts import ResourceContact
-from gajim.common.util.uri import open_uri
 
 from gajim.gtk import structs
 from gajim.gtk.alert import ConfirmationAlertDialog
@@ -69,8 +71,10 @@ from gajim.gtk.const import APP_ACTIONS
 from gajim.gtk.const import FEATURE_ACCOUNT_ACTIONS
 from gajim.gtk.const import MuteState
 from gajim.gtk.const import ONLINE_ACCOUNT_ACTIONS
-from gajim.gtk.shortcuts import ShortcutsWindow
+from gajim.gtk.const import SHORTCUTS
 from gajim.gtk.util.icons import get_icon_theme
+from gajim.gtk.util.misc import open_file
+from gajim.gtk.util.misc import open_uri
 from gajim.gtk.util.window import get_app_window
 from gajim.gtk.util.window import get_app_windows
 from gajim.gtk.util.window import open_window
@@ -115,10 +119,7 @@ class GajimApplication(Adw.Application, CoreApplication):
             ord("s"),
             GLib.OptionFlags.NONE,
             GLib.OptionArg.NONE,
-            _(
-                "Separate profile files completely "
-                "(even history database and plugins)"
-            ),
+            _("Separate profile files completely (even history database and plugins)"),
         )
 
         self.add_main_option(
@@ -246,9 +247,9 @@ class GajimApplication(Adw.Application, CoreApplication):
             app.settings.get("autoawaytime") * 60, app.settings.get("autoxatime") * 60
         )
 
-        from gajim.gtk.status_icon import StatusIcon
+        from gajim.gtk.tray_icon import TrayIcon
 
-        self.systray = StatusIcon()
+        self.systray = TrayIcon()
 
         self._add_app_actions()
         accounts = app.settings.get_accounts()
@@ -319,7 +320,6 @@ class GajimApplication(Adw.Application, CoreApplication):
     def _handle_local_options(
         self, _application: Gtk.Application, options: GLib.VariantDict
     ) -> int:
-
         if options.contains("version"):
             print(gajim.__version__)
             return 0
@@ -355,7 +355,18 @@ class GajimApplication(Adw.Application, CoreApplication):
 
         GLib.set_application_name(application_name)
 
-        self.register()
+        try:
+            self.register()
+        except GLib.Error as error:
+            quark = GLib.quark_try_string("g-io-error-quark")
+            if error.matches(quark, Gio.IOErrorEnum.TIMED_OUT):
+                sys.exit(
+                    "Timeout reached for registering the application on DBus, "
+                    "check if there is another broken Gajim process"
+                )
+            else:
+                sys.exit(f"Unable to register application on DBus: {error}")
+
         if self.get_is_remote():
             print(
                 "Gajim is already running. "
@@ -412,6 +423,9 @@ class GajimApplication(Adw.Application, CoreApplication):
             ("forget-groupchat", self._on_forget_groupchat_action),
             ("open-chat", self._on_open_chat_action),
             ("mute-chat", self._on_mute_chat_action),
+            ("save-file-as", self._on_save_file_as),
+            ("open-file", self._on_open_file_folder),
+            ("open-folder", self._on_open_file_folder),
         ]
 
         for action in actions:
@@ -441,14 +455,9 @@ class GajimApplication(Adw.Application, CoreApplication):
             ("add-contact", self._on_add_contact_account_action),
             ("services", self._on_services_action),
             ("profile", self._on_profile_action),
-            ("server-info", self._on_server_info_action),
-            ("archive", self._on_archive_action),
             ("pep-config", self._on_pep_config_action),
-            ("sync-history", self._on_sync_history_action),
-            ("blocking", self._on_blocking_action),
             ("open-event", self._on_open_event_action),
             ("mark-as-read", self._on_mark_as_read_action),
-            ("manage-roster", self._on_manage_roster_action),
             ("block-contact", self._on_block_contact),
             ("remove-contact", self._on_remove_contact),
             ("execute-command", self._on_execute_command),
@@ -473,7 +482,6 @@ class GajimApplication(Adw.Application, CoreApplication):
         action.set_enabled(state)
 
     def set_account_actions_state(self, account: str, new_state: bool = False) -> None:
-
         for action_name in ONLINE_ACCOUNT_ACTIONS:
             self.set_action_state(f"{account}-{action_name}", new_state)
 
@@ -484,11 +492,8 @@ class GajimApplication(Adw.Application, CoreApplication):
 
     def update_feature_actions_state(self, account: str) -> None:
         client = app.get_client(account)
-        mam_available = client.get_module("MAM").available
         blocking_available = client.get_module("Blocking").supported
 
-        self.set_action_state(f"{account}-archive", mam_available)
-        self.set_action_state(f"{account}-blocking", blocking_available)
         self.set_action_state(f"{account}-block-contact", blocking_available)
 
     def update_app_actions_state(self) -> None:
@@ -498,21 +503,39 @@ class GajimApplication(Adw.Application, CoreApplication):
         enabled_accounts = bool(app.settings.get_active_accounts())
         self.set_action_state("start-chat", enabled_accounts)
 
-    def _load_shortcuts(self) -> None:
-        default_path = configpaths.get("DATA") / "other" / "shortcuts.json"
-        shortcuts = load_json(default_path)
-        assert shortcuts is not None
-
+    @staticmethod
+    def get_user_shortcuts() -> dict[str, list[str]]:
         user_path = configpaths.get("MY_SHORTCUTS")
-        user_shortcuts = {}
+        user_shortcuts: dict[str, list[str]] = {}
         if user_path.exists():
             app.log("app").info("Load user shortcuts")
             user_shortcuts = load_json(user_path, default={})
 
+        return user_shortcuts
+
+    def set_user_shortcuts(self, user_shortcuts: dict[str, list[str]]) -> None:
+        user_path = configpaths.get("MY_SHORTCUTS")
+        dump_json(user_path, user_shortcuts)
+        self._load_shortcuts()
+
+    def _load_shortcuts(self) -> None:
+        shortcuts = {
+            action_name: data.accelerators for action_name, data in SHORTCUTS.items()
+        }
+        no_rebind_allowed = [
+            action_name
+            for action_name, data in SHORTCUTS.items()
+            if not data.allow_rebind
+        ]
+
+        user_shortcuts = self.get_user_shortcuts()
+        for action_name in no_rebind_allowed:
+            shortcuts.pop(action_name, None)
+
         shortcuts.update(user_shortcuts)
 
-        for action, accels in shortcuts.items():
-            self.set_accels_for_action(action, accels)
+        for action, accelerators in shortcuts.items():
+            self.set_accels_for_action(action, accelerators)
 
     def _on_feature_discovered(self, event: events.FeatureDiscovered) -> None:
         self.update_feature_actions_state(event.account)
@@ -526,7 +549,6 @@ class GajimApplication(Adw.Application, CoreApplication):
         custom_host: tuple[str, ConnectionProtocol, ConnectionType] | None,
         anonymous: bool = False,
     ) -> None:
-
         CoreApplication.create_account(
             self, account, address, password, proxy_name, custom_host, anonymous
         )
@@ -536,16 +558,9 @@ class GajimApplication(Adw.Application, CoreApplication):
         # Action must be added before account window is updated
         self.add_account_actions(account)
 
-        window = get_app_window("AccountsWindow")
-        if window is not None:
-            window.add_account(account)
-
     def enable_account(self, account: str) -> None:
         CoreApplication.enable_account(self, account)
         self.update_app_actions_state()
-        window = get_app_window("AccountsWindow")
-        if window is not None:
-            window.enable_account(account, True)
 
     def disable_account(self, account: str) -> None:
         for win in get_app_windows(account):
@@ -564,10 +579,6 @@ class GajimApplication(Adw.Application, CoreApplication):
 
         self.remove_account_actions(account)
 
-        window = get_app_window("AccountsWindow")
-        if window is not None:
-            window.remove_account(account)
-
     def _on_db_migration(self, _event: events.DBMigration) -> None:
         open_window("DBMigration")
 
@@ -598,11 +609,11 @@ class GajimApplication(Adw.Application, CoreApplication):
 
     @staticmethod
     def _on_accounts_action(_action: Gio.SimpleAction, param: GLib.Variant) -> None:
-        window = open_window("AccountsWindow")
+        window = open_window("Preferences")
 
         account = param.get_string()
         if account:
-            window.select_account(account)
+            window.show_page(account)
 
     @staticmethod
     def _on_quit_action(_action: Gio.SimpleAction, _param: GLib.Variant | None) -> None:
@@ -610,7 +621,6 @@ class GajimApplication(Adw.Application, CoreApplication):
 
     @staticmethod
     def _on_new_chat_action(_action: Gio.SimpleAction, param: GLib.Variant) -> None:
-
         jid, initial_message = param.get_strv()
         open_window(
             "StartChatDialog",
@@ -647,13 +657,6 @@ class GajimApplication(Adw.Application, CoreApplication):
     ) -> None:
         open_window("AccountWizard")
 
-    @staticmethod
-    def _on_manage_roster_action(
-        _action: Gio.SimpleAction, param: GLib.Variant
-    ) -> None:
-        account = param.get_string()
-        open_window("ManageRoster", account=account)
-
     @structs.actionmethod
     def _on_block_contact(
         self, _action: Gio.SimpleAction, params: structs.AccountJidParam
@@ -702,26 +705,6 @@ class GajimApplication(Adw.Application, CoreApplication):
         account = param.get_string()
         open_window("PEPConfig", account=account)
 
-    @staticmethod
-    def _on_archive_action(_action: Gio.SimpleAction, param: GLib.Variant) -> None:
-        account = param.get_string()
-        open_window("MamPreferences", account=account)
-
-    @staticmethod
-    def _on_blocking_action(_action: Gio.SimpleAction, param: GLib.Variant) -> None:
-        account = param.get_string()
-        open_window("BlockingList", account=account)
-
-    @staticmethod
-    def _on_sync_history_action(_action: Gio.SimpleAction, param: GLib.Variant) -> None:
-        account = param.get_string()
-        open_window("HistorySyncAssistant", account=account)
-
-    @staticmethod
-    def _on_server_info_action(_action: Gio.SimpleAction, param: GLib.Variant) -> None:
-        account = param.get_string()
-        open_window("ServerInfo", account=account)
-
     @staticmethod
     def _on_xml_console_action(
         _action: Gio.SimpleAction, _param: GLib.Variant | None
@@ -764,7 +747,7 @@ class GajimApplication(Adw.Application, CoreApplication):
     def _on_shortcuts_action(
         _action: Gio.SimpleAction, _param: GLib.Variant | None
     ) -> None:
-        ShortcutsWindow()
+        open_window("Preferences").show_page("shortcuts")
 
     @staticmethod
     def _on_features_action(
@@ -795,18 +778,15 @@ class GajimApplication(Adw.Application, CoreApplication):
         _action: Gio.SimpleAction, params: structs.OpenEventActionParams
     ) -> None:
         if params.type in ("connection-failed", "server-shutdown"):
-
             app.window.show_account_page(params.account)
 
         elif params.type in (
             "subscription-request",
             "muc-invitation",
         ):
-
             app.window.show_activity_page(params.context_id)
 
         elif params.type in ("incoming-message", "incoming-call", "file-transfer"):
-
             assert params.jid
             jid = JID.from_string(params.jid)
             app.window.select_chat(params.account, jid)
@@ -817,7 +797,6 @@ class GajimApplication(Adw.Application, CoreApplication):
     def _on_mark_as_read_action(
         self, _action: Gio.SimpleAction, params: structs.AccountJidParam
     ) -> None:
-
         app.window.mark_as_read(params.account, params.jid)
 
     @staticmethod
@@ -838,7 +817,6 @@ class GajimApplication(Adw.Application, CoreApplication):
     def _on_mute_chat_action(
         _action: Gio.SimpleAction, params: structs.MuteContactParam
     ) -> None:
-
         client = app.get_client(params.account)
         contact = client.get_module("Contacts").get_contact(params.jid)
         assert not isinstance(contact, ResourceContact)
@@ -867,7 +845,7 @@ class GajimApplication(Adw.Application, CoreApplication):
 
             app.window.clear_chat_list_row(params.account, params.jid)
             control = app.window.get_control()
-            if not control.is_loaded(params.account, params.jid):
+            if not control.is_chat_active(params.account, params.jid):
                 return
 
             control.reset_view()
@@ -885,11 +863,10 @@ class GajimApplication(Adw.Application, CoreApplication):
     def _on_forget_groupchat_action(
         _action: Gio.SimpleAction, params: structs.AccountJidParam
     ) -> None:
-
         def _on_response() -> None:
             window = get_app_window("StartChatDialog")
             if window is not None:
-                window.remove_row(params.account, str(params.jid))
+                window.remove_row(params.account, params.jid)
 
             client = app.get_client(params.account)
             client.get_module("MUC").leave(params.jid)
@@ -904,3 +881,85 @@ class GajimApplication(Adw.Application, CoreApplication):
             appearance="destructive",
             callback=_on_response,
         )
+
+    def _on_save_file_as(self, _action: Gio.SimpleAction, param: GLib.Variant) -> None:
+        orig_path = Path(param.get_string())
+
+        def _on_save_finished(
+            file_dialog: Gtk.FileDialog, result: Gio.AsyncResult
+        ) -> None:
+            try:
+                gfile = file_dialog.save_finish(result)
+            except GLib.Error as e:
+                if e.code == 2:
+                    # User dismissed dialog, do nothing
+                    return
+
+                InformationAlertDialog(
+                    _("Could Not Save File"),
+                    _("Could not save file to selected directory."),
+                )
+                return
+
+            path = gfile.get_path()
+            assert path is not None
+            target_path = Path(path)
+            orig_ext = orig_path.suffix
+            new_ext = target_path.suffix
+            if orig_ext != new_ext:
+                # Windows file chooser selects the full file name including
+                # extension. Starting to type will overwrite the extension
+                # as well. Restore the extension if it's lost.
+                target_path = target_path.with_suffix(orig_ext)
+            dirname = target_path.parent
+            if not os.access(dirname, os.W_OK):
+                InformationAlertDialog(
+                    _("Directory Not Writable"),
+                    _(
+                        'Directory "%s" is not writable. '
+                        "You do not have the proper permissions to "
+                        "create files in this directory."
+                    )
+                    % dirname,
+                )
+                return
+
+            try:
+                shutil.copyfile(orig_path, target_path)
+            except Exception as e:
+                InformationAlertDialog(
+                    _("Could Not Save File"),
+                    _("There was an error while trying to save the file.\nError: %s.")
+                    % e,
+                )
+                return
+
+            app.settings.set("last_save_dir", str(target_path.parent))
+
+            app.window.show_toast(
+                Adw.Toast(
+                    title=_("File saved"),
+                    timeout=5,
+                    button_label=_("Open Folder"),
+                    action_name="app.open-folder",
+                    action_target=GLib.Variant("s", str(target_path)),
+                )
+            )
+
+        gfile = None
+        last_dir = app.settings.get("last_save_dir")
+        if last_dir:
+            gfile = Gio.File.new_for_path(last_dir)
+
+        dialog = Gtk.FileDialog(
+            initial_folder=gfile,
+            initial_name=orig_path.name,
+        )
+        dialog.save(app.window, None, _on_save_finished)
+
+    def _on_open_file_folder(
+        self, action: Gio.SimpleAction, param: GLib.Variant
+    ) -> None:
+        open_file(
+            Path(param.get_string()), show_in_folder=action.get_name() == "open-folder"
+        )
