@@ -7,8 +7,7 @@ from __future__ import annotations
 from typing import Any
 from typing import Literal
 
-from datetime import datetime
-from urllib.parse import urlparse
+import datetime as dt
 
 from gi.repository import Gdk
 from gi.repository import Gio
@@ -34,10 +33,7 @@ from gajim.common.storage.draft import DraftStorage
 from gajim.common.types import ChatContactT
 from gajim.common.util.muc import get_groupchat_name
 from gajim.common.util.muc import message_needs_highlight
-from gajim.common.util.preview import filename_from_uri
-from gajim.common.util.preview import format_geo_coords
-from gajim.common.util.preview import guess_simple_file_type
-from gajim.common.util.preview import split_geo_uri
+from gajim.common.util.preview import get_preview_data
 from gajim.common.util.user_strings import get_moderation_text
 from gajim.common.util.user_strings import get_retraction_text
 from gajim.common.util.user_strings import get_uf_relative_time
@@ -49,7 +45,6 @@ from gajim.gtk.widgets import GajimPopover
 
 
 class ChatListRow(Gtk.ListBoxRow, SignalManager):
-
     __gsignals__ = {
         "unread-changed": (GObject.SignalFlags.RUN_LAST, None, ()),
         "context-menu-state-changed": (GObject.SignalFlags.RUN_LAST, None, (bool,)),
@@ -64,7 +59,6 @@ class ChatListRow(Gtk.ListBoxRow, SignalManager):
         pinned: bool,
         position: int,
     ) -> None:
-
         Gtk.ListBoxRow.__init__(self)
         SignalManager.__init__(self)
 
@@ -265,7 +259,6 @@ class ChatListRow(Gtk.ListBoxRow, SignalManager):
         icon_name: str | None = None,
         oob: list[mod.OOB] | None = None,
     ) -> None:
-
         if self._has_draft:
             return
 
@@ -277,16 +270,10 @@ class ChatListRow(Gtk.ListBoxRow, SignalManager):
         if icon_name is not None:
             icon = Gio.Icon.new_for_string(icon_name)
 
-        if app.preview_manager.is_previewable(text, oob or []):
-            scheme = urlparse(text).scheme
-            if scheme == "geo":
-                location = split_geo_uri(text)
-                icon = Gio.Icon.new_for_string("lucide-map-pin-symbolic")
-                text = format_geo_coords(float(location.lat), float(location.lon))
-            else:
-                file_name = filename_from_uri(text)
-                icon, file_type = guess_simple_file_type(text)
-                text = f"{file_type} ({file_name})"
+        preview = get_preview_data(text, oob or [])
+        if preview is not None:
+            text = preview.text
+            icon = preview.icon
 
         text = GLib.markup_escape_text(text)
         if text.startswith("/me ") and nickname is not None:
@@ -297,6 +284,10 @@ class ChatListRow(Gtk.ListBoxRow, SignalManager):
         # Split by newline and display last line (or first, if last is newline)
         lines = text.split("\n")
         text = lines[-1] or lines[0]
+
+        if text.startswith("```"):
+            text = _("Code snippet")
+
         self._ui.message_label.set_markup(text)
 
         if icon is None:
@@ -315,7 +306,7 @@ class ChatListRow(Gtk.ListBoxRow, SignalManager):
     def set_stanza_id(self, stanza_id: str | None) -> None:
         self.stanza_id = stanza_id
 
-    def set_timestamp(self, timestamp: datetime) -> None:
+    def set_timestamp(self, timestamp: dt.datetime) -> None:
         self.timestamp = timestamp.timestamp()
         self.update_time()
 
@@ -352,9 +343,8 @@ class ChatListRow(Gtk.ListBoxRow, SignalManager):
     def update_time(self) -> None:
         if self.timestamp == 0:
             return
-        self._ui.timestamp_label.set_text(
-            get_uf_relative_time(datetime.fromtimestamp(self.timestamp))
-        )
+        utc_timestamp = dt.datetime.fromtimestamp(self.timestamp, dt.UTC)
+        self._ui.timestamp_label.set_text(get_uf_relative_time(utc_timestamp))
 
     def update_row_state(self) -> None:
         self.update_time()
@@ -420,7 +410,6 @@ class ChatListRow(Gtk.ListBoxRow, SignalManager):
         contact: ChatContactT,
         draft: Draft | None,
     ) -> None:
-
         if contact != self.contact:
             return
 
@@ -478,7 +467,6 @@ class ChatListRow(Gtk.ListBoxRow, SignalManager):
         x: float,
         y: float,
     ) -> int:
-
         if gesture_click.get_current_button() == Gdk.BUTTON_MIDDLE:
             app.window.activate_action(
                 "win.remove-chat", GLib.Variant("as", [self.account, str(self.jid)])
@@ -594,7 +582,6 @@ class ChatListRow(Gtk.ListBoxRow, SignalManager):
     def _on_mam_sync_changed(
         self, _contact: GroupchatContact, signal_name: str
     ) -> None:
-
         self._reset_connection_icon()
 
         if signal_name == "mam-sync-started":
@@ -607,7 +594,6 @@ class ChatListRow(Gtk.ListBoxRow, SignalManager):
     def _on_mam_sync_error(
         self, _contact: GroupchatContact, _signal_name: str, error_text: str
     ) -> None:
-
         self._reset_connection_icon()
         self._ui.connection_icon.set_from_icon_name("lucide-zap-symbolic")
         self._ui.connection_icon.add_css_class("error")
