@@ -49,11 +49,7 @@ STANZA_PRESETS = {
     "Message": (
         '<message to="" type="" xmlns="jabber:client">\n<body></body>\n</message>'
     ),
-    "Iq": (
-        '<iq to="" type="" xmlns="jabber:client">\n'
-        '<query xmlns=""></query>\n'
-        "</iq>"
-    ),
+    "Iq": ('<iq to="" type="" xmlns="jabber:client">\n<query xmlns=""></query>\n</iq>'),
     "XEP-0030: Disco Info Query": (
         '<iq to="" type="get" xmlns="jabber:client">\n'
         f'<query xmlns="{Namespace.DISCO_INFO}">'
@@ -211,7 +207,7 @@ class DebugConsoleWindow(GajimAppWindow, EventHelper):
     def _on_account_change(self, dropdown: GajimDropDown, *args: Any) -> None:
         item = dropdown.get_selected_item()
         assert item is not None
-        self._selected_send_account = item.props.key
+        self._selected_send_account = item.key
 
     def _set_title(self) -> None:
         if self._selected_account == "AllAccounts":
@@ -234,7 +230,6 @@ class DebugConsoleWindow(GajimAppWindow, EventHelper):
     def _on_stack_child_changed(
         self, _widget: Gtk.Stack, _pspec: GObject.ParamSpec
     ) -> None:
-
         name = self._ui.stack.get_visible_child_name()
         self._ui.search_toggle.set_sensitive(name == "protocol")
 
@@ -324,7 +319,7 @@ class DebugConsoleWindow(GajimAppWindow, EventHelper):
         if not app.account_is_available(self._selected_send_account):
             # If offline or connecting
             InformationAlertDialog(
-                _("Not Connected"),
+                _("You are offline"),
                 _('Please make sure you are connected with "%s".')
                 % self._selected_send_account,
             )
@@ -422,7 +417,6 @@ class DebugConsoleWindow(GajimAppWindow, EventHelper):
         result: Gio.AsyncResult,
         direction: Direction,
     ) -> None:
-
         if direction == Direction.NEXT:
             match = self._search_context.forward_finish(result)
         else:
