@@ -99,6 +99,12 @@ class NotificationBackend(EventHelper):
                 log.debug("Notifications muted for %s", contact)
                 return
 
+            if isinstance(contact, GroupchatContact) and event.resource is not None:
+                participant = contact.get_resource(event.resource)
+                if participant.is_blocked:
+                    log.debug("Notifications blocked for %s", participant)
+                    return
+
         if event.sound is not None:
             play_sound(event.sound, event.account)
 
@@ -120,7 +126,6 @@ class NotificationBackend(EventHelper):
     def _on_client_state_changed(
         self, client: Client, _signal_name: str, state: SimpleClientState
     ) -> None:
-
         if not state.is_connected:
             return
         self._withdraw(["connection-failed", client.account])
@@ -134,7 +139,6 @@ class NotificationBackend(EventHelper):
 
 
 class DummyBackend(NotificationBackend):
-
     def _send(self, event: events.Notification) -> None:
         pass
 
@@ -290,7 +294,6 @@ class WindowsToastNotification(NotificationBackend):
 
 
 class Linux(NotificationBackend):
-
     _action_types = [
         "connection-failed",
         "server-shutdown",
@@ -330,7 +333,6 @@ class Linux(NotificationBackend):
     def _add_actions(
         self, event: events.Notification, notification: Gio.Notification
     ) -> None:
-
         if event.type not in self._action_types:
             return
 
@@ -468,7 +470,12 @@ def _get_path_for_avatar_texture(texture: Gdk.Texture) -> Path:
 def get_notification_backend() -> NotificationBackend:
     if sys.platform == "win32":
         if int(platform.version().split(".")[2]) >= MIN_WINDOWS_TOASTS_WIN_VERSION:
-            return WindowsToastNotification()
+            try:
+                return WindowsToastNotification()
+            except OSError as e:
+                log.error(
+                    "Error while trying to initialize notification backend: %s", e
+                )
         return DummyBackend()
 
     if sys.platform == "darwin":
