@@ -31,6 +31,7 @@ from nbxmpp.structs import MuclumbusResult
 from nbxmpp.task import Task
 
 from gajim.common import app
+from gajim.common.client import Client
 from gajim.common.const import AvatarSize
 from gajim.common.const import Direction
 from gajim.common.const import MUC_DISCO_ERRORS
@@ -54,7 +55,7 @@ from gajim.gtk.chat_filter import ChatFilter
 from gajim.gtk.chat_filter import ChatFilters
 from gajim.gtk.chat_filter import ChatTypeFilter
 from gajim.gtk.groupchat_info import GroupChatInfoScrolled
-from gajim.gtk.groupchat_nick import NickChooser
+from gajim.gtk.groupchat_nick_chooser import GroupChatNickChooser
 from gajim.gtk.menus import get_start_chat_menu
 from gajim.gtk.menus import get_start_chat_row_menu
 from gajim.gtk.util.classes import SignalManager
@@ -75,13 +76,11 @@ log = logging.getLogger("gajim.gtk.start_chat")
 
 
 class StartChatDialog(GajimAppWindow):
-
     last_chat_filters = ChatFilters()
 
     def __init__(
         self, initial_jid: str | None = None, initial_message: str | None = None
     ) -> None:
-
         GajimAppWindow.__init__(
             self,
             name="StartChatDialog",
@@ -96,11 +95,12 @@ class StartChatDialog(GajimAppWindow):
         self._destroyed = False
         self._search_stopped = False
         self._search_is_changed = False
+        self._http_search_client: Client | None = None
 
         self._ui = get_builder("start_chat_dialog.ui")
         self.set_child(self._ui.stack)
 
-        self._nick_chooser = NickChooser()
+        self._nick_chooser = GroupChatNickChooser()
         self._ui.join_box.prepend(self._nick_chooser)
 
         self._search_is_valid_jid = False
@@ -217,6 +217,7 @@ class StartChatDialog(GajimAppWindow):
         del self._muc_info_box
         del self._chat_filter
         del self._accounts_store
+        del self._http_search_client
         self._new_contact_items.clear()
         self._destroyed = True
         app.cancel_tasks(self)
@@ -378,7 +379,6 @@ class StartChatDialog(GajimAppWindow):
         _keycode: int,
         state: Gdk.ModifierType,
     ) -> bool:
-
         if keyval == Gdk.KEY_Down:
             self._ui.search_entry.emit("next-match")
             return Gdk.EVENT_STOP
@@ -434,7 +434,6 @@ class StartChatDialog(GajimAppWindow):
         _keycode: int,
         state: Gdk.ModifierType,
     ) -> bool:
-
         if keyval == Gdk.KEY_Escape:
             if self._ui.stack.get_visible_child_name() == "progress":
                 # Propagate to GajimAppWindow
@@ -529,7 +528,7 @@ class StartChatDialog(GajimAppWindow):
 
     def _disco_info(self, item: ContactListItem) -> None:
         if not app.account_is_available(item.account):
-            self._show_error_page(_("You are offline."))
+            self._show_error_page(_("You are offline"))
             return
 
         self._ui.stack.set_visible_child_name("progress")
@@ -686,7 +685,6 @@ class StartChatDialog(GajimAppWindow):
             item.props.jid = JID.from_string(search_text)
 
     def _select_new_match(self, _entry: Gtk.Entry, direction: Direction) -> None:
-
         if self._is_global_search_active():
             self._global_search_view.select(direction)
         else:
@@ -706,7 +704,7 @@ class StartChatDialog(GajimAppWindow):
         self._global_search_view.start_search()
 
         if app.settings.get("muclumbus_api_pref") == "http":
-            self._start_http_search(client, text)
+            self._start_http_search(app.get_client(accounts[0]), text)
         else:
             self._start_iq_search(client, text)
 
@@ -744,27 +742,27 @@ class StartChatDialog(GajimAppWindow):
 
         self._global_search_view.end_search()
 
-    @as_task
-    def _start_http_search(self, client: NBXMPPClient, text: str):
-        _task = yield  # noqa: F841
-
+    def _start_http_search(self, client: Client, text: str) -> None:
         self._keywords = text.split(" ")
-        result = yield client.get_module("Muclumbus").set_http_search(
-            app.settings.get("muclumbus_api_http_uri"), self._keywords
+        self._http_search_client = client
+        client.get_module("HttpMucSearch").search(
+            self._keywords, None, self._on_http_result
         )
 
-        self._process_search_result(result)
-
-        while not result.end:
-            result = yield client.get_module("Muclumbus").set_http_search(
-                app.settings.get("muclumbus_api_http_uri"),
-                self._keywords,
-                after=result.last,
-            )
-
+    def _on_http_result(self, result: MuclumbusResult) -> None:
+        try:
             self._process_search_result(result)
+        except CancelledError:
+            return
 
-        self._global_search_view.end_search()
+        if result.end:
+            self._global_search_view.end_search()
+            return
+
+        assert self._http_search_client is not None
+        self._http_search_client.get_module("HttpMucSearch").search(
+            self._keywords, result.last, self._on_http_result
+        )
 
     def _process_search_result(
         self, result: MuclumbusResult, parameters: bool = False
@@ -786,7 +784,6 @@ class StartChatDialog(GajimAppWindow):
 
 
 class BaseListView(Generic[L, V], Gtk.ListView, SignalManager):
-
     _selection_model: Gtk.SingleSelection
     _filter_model: Gtk.FilterListModel
 
@@ -909,7 +906,6 @@ class ContactListView(BaseListView[type["ContactListItem"], type["ContactViewIte
     def _on_filter_items_changed(
         self, filter_model: Gtk.FilterListModel, _pos: int, _removed: int, _added: int
     ) -> None:
-
         # Cancel any active source at first so we dont have
         # multiple timeouts running
 
@@ -961,7 +957,6 @@ class ContactListView(BaseListView[type["ContactListItem"], type["ContactViewIte
         obj2: ContactListItem,
         _user_data: object | None,
     ) -> int:
-
         if obj1.is_new != obj2.is_new:
             return 1 if obj1.is_new else -1
 
@@ -1030,7 +1025,6 @@ class ContactListItem(GObject.Object):
         account_visible: bool,
         groupchat: bool = False,
     ) -> None:
-
         name = name or _("Start / Join Chat")
 
         idle = None
@@ -1155,7 +1149,6 @@ class ContactViewItem(Gtk.Grid, SignalManager):
         x: float,
         y: float,
     ) -> int:
-
         if self._menu.get_menu_model() is None:
             return Gdk.EVENT_PROPAGATE
 
@@ -1170,7 +1163,6 @@ class GlobalSearch(
     Gtk.ListView,
     SignalManager,
 ):
-
     __gsignals__ = {
         "global-search-progress": (GObject.SignalFlags.RUN_FIRST, None, (bool, int)),
     }
